@@ -1,6 +1,7 @@
 import datetime
 import gzip
 import io
+import os
 import pickle
 import threading
 from textwrap import dedent
@@ -10,6 +11,7 @@ from urllib.parse import urlparse
 
 import boto3
 import boto3.s3.transfer
+from botocore.config import Config as ClientConfig
 from botocore.exceptions import ClientError
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
@@ -41,6 +43,18 @@ class S3StorageTests(TestCase):
                 _ = storage.connection
                 mock_session.assert_called_once_with(profile_name="test_profile")
 
+    def test_client_config(self):
+        with override_settings(
+            AWS_S3_CLIENT_CONFIG=ClientConfig(max_pool_connections=30)
+        ):
+            storage = s3.S3Storage()
+            with mock.patch("boto3.Session.resource") as mock_resource:
+                _ = storage.connection
+                mock_resource.assert_called_once()
+                self.assertEqual(
+                    30, mock_resource.call_args[1]["config"].max_pool_connections
+                )
+
     def test_pickle_with_bucket(self):
         """
         Test that the storage can be pickled with a bucket attached
@@ -299,10 +313,9 @@ class S3StorageTests(TestCase):
         Test opening a file in "r" mode (ie reading as string, not bytes)
         """
         name = "test_open_read_string.txt"
-        file = self.storage.open(name, "r")
-        content_str = file.read()
-        self.assertEqual(content_str, "")
-        file.close()
+        with self.storage.open(name, "r") as file:
+            content_str = file.read()
+            self.assertEqual(content_str, "")
 
     def test_storage_open_readlines(self):
         """
@@ -311,17 +324,15 @@ class S3StorageTests(TestCase):
         name = "test_open_readlines_string.txt"
         with io.BytesIO() as temp_file:
             temp_file.write(b"line1\nline2")
-            file = self.storage.open(name, "r")
-            file._file = temp_file
-
-            content_lines = file.readlines()
-            self.assertEqual(content_lines, ["line1\n", "line2"])
-
-            temp_file.seek(0)
-            file = self.storage.open(name, "rb")
-            file._file = temp_file
-            content_lines = file.readlines()
-            self.assertEqual(content_lines, [b"line1\n", b"line2"])
+            with self.storage.open(name, "r") as f1:
+                f1._file = temp_file
+                content_lines = f1.readlines()
+                self.assertEqual(content_lines, ["line1\n", "line2"])
+                temp_file.seek(0)
+                with self.storage.open(name, "rb") as f2:
+                    f2._file = temp_file
+                    content_lines = f2.readlines()
+                    self.assertEqual(content_lines, [b"line1\n", b"line2"])
 
     def test_storage_open_write(self):
         """
@@ -337,26 +348,24 @@ class S3StorageTests(TestCase):
             "ACL": "public-read",
         }
 
-        file = self.storage.open(name, "w")
-        self.storage.bucket.Object.assert_called_with(name)
-        obj = self.storage.bucket.Object.return_value
-        # Set the name of the mock object
-        obj.key = name
+        with self.storage.open(name, "w") as file:
+            self.storage.bucket.Object.assert_called_with(name)
+            obj = self.storage.bucket.Object.return_value
+            # Set the name of the mock object
+            obj.key = name
 
-        multipart = obj.initiate_multipart_upload.return_value
-        multipart.Part.return_value.upload.side_effect = [
-            {"ETag": "123"},
-        ]
-        file.write(content)
-        obj.initiate_multipart_upload.assert_called_with(
-            ACL="public-read",
-            ContentType="text/plain",
-            ServerSideEncryption="AES256",
-            StorageClass="REDUCED_REDUNDANCY",
-        )
+            multipart = obj.initiate_multipart_upload.return_value
+            multipart.Part.return_value.upload.side_effect = [
+                {"ETag": "123"},
+            ]
+            file.write(content)
+            obj.initiate_multipart_upload.assert_called_with(
+                ACL="public-read",
+                ContentType="text/plain",
+                ServerSideEncryption="AES256",
+                StorageClass="REDUCED_REDUNDANCY",
+            )
 
-        # Save the internal file before closing
-        file.close()
         multipart.Part.assert_called_with(1)
         part = multipart.Part.return_value
         part.upload.assert_called_with(Body=content.encode())
@@ -368,13 +377,12 @@ class S3StorageTests(TestCase):
         """Test that bytearray write exactly (no extra "bytearray" from stringify)."""
         name = "saved_file.bin"
         content = bytearray(b"content")
-        file = self.storage.open(name, "wb")
-        obj = self.storage.bucket.Object.return_value
-        # Set the name of the mock object
-        obj.key = name
-        bytes_written = file.write(content)
-        self.assertEqual(len(content), bytes_written)
-        file.close()
+        with self.storage.open(name, "wb") as file:
+            obj = self.storage.bucket.Object.return_value
+            # Set the name of the mock object
+            obj.key = name
+            bytes_written = file.write(content)
+            self.assertEqual(len(content), bytes_written)
 
     def test_storage_open_no_write(self):
         """
@@ -390,18 +398,16 @@ class S3StorageTests(TestCase):
             "StorageClass": "REDUCED_REDUNDANCY",
         }
 
-        file = self.storage.open(name, "w")
-        self.storage.bucket.Object.assert_called_with(name)
-        obj = self.storage.bucket.Object.return_value
-        obj.load.side_effect = ClientError(
-            {"Error": {}, "ResponseMetadata": {"HTTPStatusCode": 404}}, "head_bucket"
-        )
+        with self.storage.open(name, "w"):
+            self.storage.bucket.Object.assert_called_with(name)
+            obj = self.storage.bucket.Object.return_value
+            obj.load.side_effect = ClientError(
+                {"Error": {}, "ResponseMetadata": {"HTTPStatusCode": 404}},
+                "head_bucket",
+            )
 
-        # Set the name of the mock object
-        obj.key = name
-
-        # Save the internal file before closing
-        file.close()
+            # Set the name of the mock object
+            obj.key = name
 
         obj.load.assert_called_once_with()
         obj.put.assert_called_once_with(
@@ -423,15 +429,12 @@ class S3StorageTests(TestCase):
             "StorageClass": "REDUCED_REDUNDANCY",
         }
 
-        file = self.storage.open(name, "w")
-        self.storage.bucket.Object.assert_called_with(name)
-        obj = self.storage.bucket.Object.return_value
+        with self.storage.open(name, "w"):
+            self.storage.bucket.Object.assert_called_with(name)
+            obj = self.storage.bucket.Object.return_value
 
-        # Set the name of the mock object
-        obj.key = name
-
-        # Save the internal file before closing
-        file.close()
+            # Set the name of the mock object
+            obj.key = name
 
         obj.load.assert_called_once_with()
         obj.put.assert_not_called()
@@ -448,39 +451,37 @@ class S3StorageTests(TestCase):
             "StorageClass": "REDUCED_REDUNDANCY",
         }
 
-        file = self.storage.open(name, "w")
-        self.storage.bucket.Object.assert_called_with(name)
-        obj = self.storage.bucket.Object.return_value
-        # Set the name of the mock object
-        obj.key = name
+        with self.storage.open(name, "w") as file:
+            self.storage.bucket.Object.assert_called_with(name)
+            obj = self.storage.bucket.Object.return_value
+            # Set the name of the mock object
+            obj.key = name
 
-        # Initiate the multipart upload
-        file.write("")
-        obj.initiate_multipart_upload.assert_called_with(
-            ContentType="text/plain",
-            ServerSideEncryption="AES256",
-            StorageClass="REDUCED_REDUNDANCY",
-        )
-        multipart = obj.initiate_multipart_upload.return_value
+            # Initiate the multipart upload
+            file.write("")
+            obj.initiate_multipart_upload.assert_called_with(
+                ContentType="text/plain",
+                ServerSideEncryption="AES256",
+                StorageClass="REDUCED_REDUNDANCY",
+            )
+            multipart = obj.initiate_multipart_upload.return_value
 
-        # Write content at least twice as long as the buffer size
-        written_content = ""
-        counter = 1
-        multipart.Part.return_value.upload.side_effect = [
-            {"ETag": "123"},
-            {"ETag": "456"},
-        ]
-        while len(written_content) < 2 * file.buffer_size:
-            content = "hello, aws {counter}\n".format(counter=counter)
-            # Write more than just a few bytes in each iteration to keep the
-            # test reasonably fast
-            content += "*" * int(file.buffer_size / 10)
-            file.write(content)
-            written_content += content
-            counter += 1
+            # Write content at least twice as long as the buffer size
+            written_content = ""
+            counter = 1
+            multipart.Part.return_value.upload.side_effect = [
+                {"ETag": "123"},
+                {"ETag": "456"},
+            ]
+            while len(written_content) < 2 * file.buffer_size:
+                content = "hello, aws {counter}\n".format(counter=counter)
+                # Write more than just a few bytes in each iteration to keep the
+                # test reasonably fast
+                content += "*" * int(file.buffer_size / 10)
+                file.write(content)
+                written_content += content
+                counter += 1
 
-        # Save the internal file before closing
-        file.close()
         self.assertListEqual(
             multipart.Part.call_args_list, [mock.call(1), mock.call(2)]
         )
@@ -890,6 +891,16 @@ class S3StorageTests(TestCase):
             storage = s3.S3Storage()
             self.assertIsNotNone(storage.cloudfront_signer)
 
+            # allow disabling cloudfront signing
+            storage = s3.S3Storage(cloudfront_signer=None)
+            self.assertIsNone(storage.cloudfront_signer)
+
+            # allow disabling cloudfront signing in subclass
+            class Storage(s3.S3Storage):
+                cloudfront_signer = None
+
+            self.assertIsNone(Storage().cloudfront_signer)
+
         storage = s3.S3Storage(cloudfront_key_id=key_id, cloudfront_key=pem)
         self.assertIsNotNone(storage.cloudfront_signer)
 
@@ -934,6 +945,21 @@ class S3StorageTests(TestCase):
             self.assertEqual(storage.access_key, "foo")
             self.assertEqual(storage.secret_key, "boo")
 
+        with mock.patch.dict(
+            os.environ,
+            {"AWS_S3_ACCESS_KEY_ID": "foo", "AWS_S3_SECRET_ACCESS_KEY": "boo"},
+        ):
+            storage = s3.S3Storage()
+            self.assertEqual(storage.access_key, "foo")
+            self.assertEqual(storage.secret_key, "boo")
+
+        with mock.patch.dict(
+            os.environ, {"AWS_ACCESS_KEY_ID": "foo", "AWS_SECRET_ACCESS_KEY": "boo"}
+        ):
+            storage = s3.S3Storage()
+            self.assertEqual(storage.access_key, "foo")
+            self.assertEqual(storage.secret_key, "boo")
+
         storage = s3.S3Storage(access_key="foo", secret_key="boo")
         self.assertEqual(storage.access_key, "foo")
         self.assertEqual(storage.secret_key, "boo")
@@ -999,23 +1025,22 @@ class S3FileTests(TestCase):
         )
 
     def test_closed(self):
-        f = s3.S3File("test", "wb", self.storage)
+        with s3.S3File("test", "wb", self.storage) as f:
+            with self.subTest("after init"):
+                self.assertFalse(f.closed)
 
-        with self.subTest("after init"):
-            self.assertFalse(f.closed)
+            with self.subTest("after file access"):
+                # Ensure _get_file has been called
+                f.file
+                self.assertFalse(f.closed)
 
-        with self.subTest("after file access"):
-            # Ensure _get_file has been called
-            f.file
-            self.assertFalse(f.closed)
+            with self.subTest("after close"):
+                f.close()
+                self.assertTrue(f.closed)
 
-        with self.subTest("after close"):
-            f.close()
-            self.assertTrue(f.closed)
-
-        with self.subTest("reopening"):
-            f.file
-            self.assertFalse(f.closed)
+            with self.subTest("reopening"):
+                f.file
+                self.assertFalse(f.closed)
 
     def test_reopening(self):
         f = s3.S3File("test", "wb", self.storage)
